@@ -4,16 +4,31 @@ Loads data from the crest spreadsheet.
 """
 
 from datetime import time, timedelta
-from demod.utils.error_messages import NOT_IMPLEMENTED_IN_DATASET_FOR_VERSION, UNKOWN_POPULATION_TYPE
+from sys import version
+import shutil
 import warnings
 import os
+from urllib import request
 from typing import Any, List, Tuple, Dict, Union
 
 import pandas as pd
 import numpy as np
 
-from ..base_loader import ApplianceLoader, ClimateLoader, DatasetLoader, LightingLoader, PopulationLoader
-from ...utils.sim_types import AppliancesDict, StateLabels, Subgroup, Subgroups, TPMs
+from ..base_loader import (
+    ApplianceLoader,
+    ClimateLoader,
+    DatasetLoader,
+    HeatingLoader,
+    LightingLoader,
+    PopulationLoader,
+)
+from ...utils.sim_types import (
+    AppliancesDict,
+    StateLabels,
+    Subgroup,
+    Subgroups,
+    TPMs,
+)
 from ...utils.subgroup_handling import (
     check_weekend_day_format,
     is_weekday,
@@ -21,34 +36,46 @@ from ...utils.subgroup_handling import (
     subgroup_string,
 )
 from ...utils.distribution_functions import rescale_pdf
-from ...utils.parse_helpers import translate_1d
+from ...utils.parse_helpers import remove_spaces, translate_1d, bulbs_stats_from_config
 from ..tou_loader import LoaderTOU
 from .utils import (
     crest_act_to_demod,
     crest_appname_to_demod_type,
 )
+from ...utils.error_messages import (
+    DATASET_CANNOT_DISTINGUISH_ON_SUBGROUPS,
+    NOT_IMPLEMENTED_IN_DATASET_FOR_VERSION,
+    UNKOWN_POPULATION_TYPE,
+)
 
 
+DOWNLOAD_URL_BY_VERSION = {
+    '2.2': 'https://repository.lboro.ac.uk/ndownloader/files/4969096',
+    '2.3.3': 'https://repository.lboro.ac.uk/ndownloader/files/22693271',
+}
+
 class Crest(
-    ApplianceLoader,
-    LoaderTOU,
-    ClimateLoader,
-    LightingLoader,
-    PopulationLoader
-    ):
+    ApplianceLoader, LoaderTOU, ClimateLoader, LightingLoader,
+    PopulationLoader, HeatingLoader
+):
     """Crest Data.
 
+    CREST model was developped by Loughborough University, Leicestershire, UK
+
+    The CREST demand model can be accessed
+    `here
+    <https://repository.lboro.ac.uk/articles/dataset/CREST_Demand_Model_v2_0/2001129>`_
+
     Args:
         version: The version of CREST to use. Defaults to '2.2'.
-        allow_pickle: [description]. Defaults to False.
     """
 
     DATASET_NAME = "CREST"
     refresh_time = time(0, 0, 0)
+    # Step size of the load simulation
+    step_size = timedelta(minutes=1)
 
-    def __init__(
-        self, version: str = "2.2", /, **kwargs
-    ) -> Any:
+    def __init__(self, version: str = "2.2", /, **kwargs) -> Any:
         """Create a data loader for CREST.
 
         Args:
@@ -60,50 +87,76 @@ class Crest(
         """
         version_name = "v_" + version
         self.version = version
-        super().__init__(version=version_name,  **kwargs)
+        super().__init__(version=version_name, **kwargs)
 
         raw_file_name = "CREST_Demand_Model_v" + version + ".xlsm"
         self.raw_file_path = os.path.join(self.raw_path, raw_file_name)
 
+
+        # Downloads the raw file if it does not exist
+        if not os.path.isfile(self.raw_file_path):
+            if version not in DOWNLOAD_URL_BY_VERSION:
+                raise ValueError('Unkonw Version {} of CREST.'.format(version))
+            # Reads the url and  download the
+            print('Downloading CREST Model to : {}'.format(self.raw_file_path))
+            with request.urlopen(DOWNLOAD_URL_BY_VERSION[version]) as response:
+                with open(self.raw_file_path, 'wb') as f:
+                    shutil.copyfileobj(response, f)
+
+    def load_population_subgroups(
+        self, population_type: str = "crest",
+    ) -> Tuple[Subgroups, List[float], int]:
+        # Defaults the population type
+        return super().load_population_subgroups(population_type)
+
     def _parse_population_subgroups(
         self, population_type: str
     ) -> Tuple[Subgroups, List[float], int]:
-        """Read crest data to get the distribution of the population
+        """Parse the subgroups.
+
+        Read crest data to get the distribution of the population
         in dwellings depending on the number of residents in them.
         """
-        if population_type == 'crest' or population_type == 'resident_number':
+        if population_type == "crest" or population_type == "resident_number":
             try:
                 df = pd.read_excel(
-                    self.raw_file_path, 'ActivityStats',
-                    header=11, nrows=5, usecols=[1, 2], engine="openpyxl"
+                    self.raw_file_path,
+                    "ActivityStats",
+                    header=11,
+                    nrows=5,
+                    usecols=[1, 2],
+                    engine="openpyxl",
                 )
             except FileNotFoundError:
                 self._raise_missing_raw()
 
-            if self.version == 'v_2.3.3':
+            if self.version == "v_2.3.3":
+                pdf = df[(
+                    "Probability of dwelling having this number of residents,"
+                    " in the UK"
+                )].to_numpy()
+            elif self.version == "v_2.2":
                 pdf = df[
-                    "Probability of dwelling having this number of residents, in the UK"
-                ].to_numpy()
-            elif self.version == 'v_2.2':
-                pdf = df[
-                    'Probability of dwelling having this number of residents'
+                    "Probability of dwelling having this number of residents"
                 ].to_numpy()
             else:
-                raise ValueError(NOT_IMPLEMENTED_IN_DATASET_FOR_VERSION.format(
-                    not_implemented='_parse_population_subgroups',
-                    dataset=self,
-                    version=self.version,
-                ))
+                raise ValueError(
+                    NOT_IMPLEMENTED_IN_DATASET_FOR_VERSION.format(
+                        not_implemented="_parse_population_subgroups",
+                        dataset=self,
+                        version=self.version,
+                    )
+                )
             n_of_residents = df["Number of residents"].to_numpy()
 
-            subgroups = [{'n_residents': n} for n in n_of_residents]
+            subgroups = [{"n_residents": n} for n in n_of_residents]
 
         else:
-            raise ValueError(UNKOWN_POPULATION_TYPE.format(
-                population_type, self
-            ))
+            raise ValueError(
+                UNKOWN_POPULATION_TYPE.format(population_type, self)
+            )
 
-        return subgroups, pdf, 25e6 # estimate
+        return subgroups, pdf, 25e6  # estimate
 
     def _raise_missing_raw(self, *args, **kwargs):
         return super()._raise_missing_raw(
@@ -276,13 +329,14 @@ class Crest(
             associated_activity_use_profile
         )
 
-
-        appliances['inactive_switch_off'] = np.where(
+        appliances["inactive_switch_off"] = np.where(
             np.logical_or(
-                associated_activity_use_profile == 'ACT_WASHDRESS',
-                associated_activity_use_profile == 'LEVEL',
-                appliances['name'] == 'DISH_WASHER',
-            ), False, True
+                associated_activity_use_profile == "ACT_WASHDRESS",
+                associated_activity_use_profile == "LEVEL",
+                appliances["name"] == "DISH_WASHER",
+            ),
+            False,
+            True,
         )
 
         appliances["mean_duration"] = np.array(df["(min).1"])
@@ -391,14 +445,15 @@ class Crest(
         )
         app_ownership = {}
         for app_type, ownership in zip(
-                crest_dic["type"], crest_dic["equipped_prob"]):
+            crest_dic["type"], crest_dic["equipped_prob"]
+        ):
 
             # checks for multiply represented appliances
             counter = 1
             temp_type = app_type
             while temp_type in app_ownership:
                 counter += 1
-                temp_type = app_type + '_' + str(counter)
+                temp_type = app_type + "_" + str(counter)
 
             app_ownership[temp_type] = ownership
 
@@ -420,11 +475,12 @@ class Crest(
             step_size = timedelta(minutes=1)
 
             labels = np.array(df.columns)
-            labels[labels=='Clear'] = 1.
+            labels[labels == "Clear"] = 1.0
             labels = np.array(labels, dtype=float)
 
             clearness_tpm = df.to_numpy()
-            # corrects a bit the values of clearness TPM for round off erros and non existing state
+            # corrects a bit the values of clearness TPM for round off erros
+            # and non existing state
             clearness_tpm[0, 1] = 1
             clearness_tpm = rescale_pdf(clearness_tpm)
 
@@ -439,19 +495,19 @@ class Crest(
     def _parse_geographic_data(self) -> Dict[str, Union[str, float]]:
         if self.version == "v_2.2":
             return {
-                'country': 'england',
-                'latitude': 52.8,
-                'longitude': -1.2,
-                'meridian': 0.,
-                'use_daylight_saving_time': True,
+                "country": "england",
+                "latitude": 52.8,
+                "longitude": -1.2,
+                "meridian": 0.0,
+                "use_daylight_saving_time": True,
             }
         elif self.version == "v_2.3.3":
             return {
-                'country': 'india',
-                'latitude': 13.1,
-                'longitude': 80.3,
-                'meridian': 82.5,
-                'use_daylight_saving_time': False,
+                "country": "india",
+                "latitude": 13.1,
+                "longitude": 80.3,
+                "meridian": 82.5,
+                "use_daylight_saving_time": False,
             }
         else:
             raise NotImplementedError(
@@ -460,14 +516,14 @@ class Crest(
             )
 
     def _parse_temperatures_arma(self) -> Dict[str, float]:
-        if self.version == 'v_2.2':
+        if self.version == "v_2.2":
             arma_dic = {
-                'T_mean': 9.3,
-                'T_std': 6.5,
-                'T_shift': -115,
-                'AR': 0.81,
-                'MA': 0.62,
-                'SD': 0.5,
+                "T_mean": 9.3,
+                "T_std": 6.5,
+                "T_shift": -115,
+                "AR": 0.81,
+                "MA": 0.62,
+                "SD": 0.5,
             }
         else:
             raise NotImplementedError(
@@ -480,16 +536,17 @@ class Crest(
         if len(subgroup) > 0:
             # Warns that subgroup won't be used.
             warnings.warn(
-                'Crest does not distinguish bulbs config'
-                'based on subgroups.'
+                "Crest does not distinguish bulbs config" "based on subgroups."
             )
-        if self.version in ('v_2.2', 'v_2.3.3'):
+        if self.version in ("v_2.2", "v_2.3.3"):
             df = pd.read_excel(
-                self.raw_file_path, header=9,
-                sheet_name="bulbs", engine="openpyxl",
+                self.raw_file_path,
+                header=9,
+                sheet_name="bulbs",
+                engine="openpyxl",
             )
             # Select the columns of interest
-            bulbs_config =  df.to_numpy()[:100, 2:40]
+            bulbs_config = df.to_numpy()[:100, 2:40]
         else:
             raise NotImplementedError(
                 "'_parse_bulbs_config' is not implemented for CREST "
@@ -507,10 +564,10 @@ class Crest(
             engine="openpyxl",
         )
 
-        if self.version == 'v_2.2':
-            crest_dict['calibration_scalar'] = float(df.columns[5])
-        elif self.version == 'v_2.3.3':
-            crest_dict['calibration_scalar'] = float(df.columns[6])
+        if self.version == "v_2.2":
+            crest_dict["calibration_scalar"] = float(df.columns[5])
+        elif self.version == "v_2.3.3":
+            crest_dict["calibration_scalar"] = float(df.columns[6])
         else:
             raise NotImplementedError(
                 "'_parse_crest_lighting' is not implemented for CREST "
@@ -519,28 +576,28 @@ class Crest(
 
         df = pd.read_excel(
             self.raw_file_path,
-            header=35, sheet_name="light_config",
+            header=35,
+            sheet_name="light_config",
             engine="openpyxl",
         )
-        crest_dict['effective_occupancy'] = (
-            np.array(df["occupancy"][:6], dtype=float)
+        crest_dict["effective_occupancy"] = np.array(
+            df["occupancy"][:6], dtype=float
         )
 
-
         df = pd.read_excel(
             self.raw_file_path,
             sheet_name="light_config",
             header=53,
             engine="openpyxl",
         )
-        crest_dict['durations_cdf'] = (
-            np.array(df["probability"][:9], dtype=float)
+        crest_dict["durations_cdf"] = np.array(
+            df["probability"][:9], dtype=float
         )
-        crest_dict['durations_minutes_low'] = (
-            np.array(df["(minutes)"][:9], dtype=float)
+        crest_dict["durations_minutes_low"] = np.array(
+            df["(minutes)"][:9], dtype=float
         )
-        crest_dict['durations_minutes_high'] = (
-            np.array(df["(minutes).1"][:9], dtype=float)
+        crest_dict["durations_minutes_high"] = np.array(
+            df["(minutes).1"][:9], dtype=float
         )
 
         df = pd.read_excel(
@@ -549,8 +606,265 @@ class Crest(
             header=3,
             engine="openpyxl",
         )
-        crest_dict['irradiance_threshold_mean'] = float(df.columns[5])
-        crest_dict['irradiance_threshold_std'] = float(df.columns[6])
+        crest_dict["irradiance_threshold_mean"] = float(df.columns[5])
+        crest_dict["irradiance_threshold_std"] = float(df.columns[6])
 
         return crest_dict
 
+    def _parse_installed_bulbs_stats(self, subgroup: Subgroup):
+        warnings.warn(
+            DATASET_CANNOT_DISTINGUISH_ON_SUBGROUPS.format(
+                dataset=self,
+                not_distinguishable='bulbs stats',
+            )
+        )
+        stats, _ = bulbs_stats_from_config(self.load_bulbs_config(subgroup))
+        return stats
+
+    def _parse_bulbs(self):
+        _, bulbs = bulbs_stats_from_config(self.load_bulbs_config())
+        return bulbs
+
+    def _parse_heating_system_dict(self, subgroup: Subgroup = {}):
+        warnings.warn(
+            DATASET_CANNOT_DISTINGUISH_ON_SUBGROUPS.format(
+                dataset=self,
+                not_distinguishable='heating system',
+            )
+        )
+        if self.version == "v_2.2":
+            rows = 3
+        elif self.version == "v_2.3.3":
+            rows = 5
+        else:
+            raise NotImplementedError(
+                NOT_IMPLEMENTED_IN_DATASET_FOR_VERSION.format(
+                    not_implemented=self._parse_heating_system_dict,
+                    dataset=self,
+                    version=self.version
+                )
+            )
+
+        df = pd.read_excel(
+            self.raw_file_path,
+            sheet_name='PrimaryHeatingSystems',
+            header=1, nrows=rows, skiprows=[2, 3]
+        )
+
+        heating_systems = {}
+        heating_systems["equipped_prob"] = np.array(
+             df["Proportion of dwellings with this heating system"]
+        )
+        heating_systems["name"] = np.array(df["Type of heating unit"])
+        heating_systems["is_combi"] = np.array(df["Type of system"]) == 2
+        heating_systems["fuel_type"] = remove_spaces(df["Type of fuel"])
+        heating_systems["fuel_flow_rate"] = np.array(
+            df["Fuel flow rate (nominal)"]
+        )
+        heating_systems["flow_rate_to_W"] = np.array(
+            df["Calorific value of fuel flow rate"]
+        )
+        # Same heat outputs sh and dhw
+        heating_systems["heat_output_sh"] = np.array(
+            df["Heat output of unit"]
+        )
+        heating_systems["heat_output_dhw"] = np.array(
+            df["Heat output of unit"]
+        )
+        heating_systems["standby_power"] = np.array(df["Standby power"])
+        heating_systems["pump_power"] = np.array(df["Pump power"])
+        heating_systems["cyl_volume"] = np.array(
+            df["DHW cylinder volume"]
+        )
+        heating_systems["cyl_loss"] = np.array(
+            df["DHW Tank Heat Loss"]
+        )
+
+        return heating_systems
+
+
+    def _parse_thermostat_dict(self, subgroup: Subgroup):
+        warnings.warn(
+            DATASET_CANNOT_DISTINGUISH_ON_SUBGROUPS.format(
+                dataset=self,
+                not_distinguishable='thermostat dict',
+            )
+        )
+
+        df_home = pd.read_excel(
+            self.raw_file_path,
+            sheet_name='HeatingControls',
+            header=3, nrows=15
+        )
+        df_water = pd.read_excel(
+            self.raw_file_path,
+            sheet_name='HeatingControls',
+            header=22, nrows=12
+        )
+
+        thermostat = {}
+        thermostat["home_temperatures_cdf"] = np.cumsum(
+            np.array(df_home["Percentage of homes"])
+        )
+        thermostat["home_temperatures_values"] = np.array(
+            df_home["Demand temperature"], dtype=float
+        )
+        thermostat["water_temperatures_cdf"] = np.cumsum(
+            np.array(df_water["Percentage of homes"])
+        )
+        thermostat["water_temperatures_values"] = np.array(
+            df_water["Hot water delivery temperature"], dtype=float
+        )
+
+
+
+        # crest value for the emitters
+        thermostat["emitter_setpoints"] = 50.0
+
+        # Set thermostat deadbands
+        deadbands = {}
+        deadbands["space_heating"] = 2
+        deadbands["space_cooling"] = 2
+        deadbands["hot_water"] = 5
+        deadbands["emitter"] = 5
+        thermostat["deadband"] = deadbands
+
+        return thermostat
+
+    def _parse_buildings_dict(self, subgroup: Subgroup):
+
+        df = pd.read_excel(
+            self.raw_file_path,
+            sheet_name='Buildings',
+            header=1, skiprows=[2, 3]
+        )
+
+        buildings = {}
+        buildings["equipped_prob"] = np.array(
+            df["Proportion of dwellings of this building type"]
+        )
+        buildings["name"] = np.array(df["Description"])
+        buildings["out_build_transfer_coef"] = np.array(
+            df[
+                "Thermal transfer coefficient between outside air and external building thermal capacitance"
+            ]
+        )
+        buildings["build_int_transfer_coef"] = np.array(
+            df[
+                "Thermal transfer coefficient between external building thermal capacitance and internal building thermal capacitance"
+            ]
+        )
+        buildings["ventilation_transfer_coef"] = np.array(
+            df[
+                "Thermal transfer coefficient representing ventilation heat loss between outside air and internal building thermal capacitance"
+            ]
+        )
+        buildings["ext_capacitance"] = np.array(
+            df["External building thermal capacitance"]
+        )
+        buildings["int_capacitance"] = np.array(
+            df["Internal building thermal capacitance"]
+        )
+        buildings["irradiance_multiplier"] = np.array(
+            df["Global irradiance multiplier"]
+        )
+        buildings["floor_area"] = np.array(df["Floor area, living space"])
+        buildings["height"] = np.array(df["Height, living space"])
+        print(df.columns)
+        buildings["emitters_target_temperature"] = np.array(
+            df["Nominal temperature of emitters "]
+        )
+        buildings["emitters_transfer_coef"] = np.array(
+            df["Heat transfer coefficient of emitters"]
+            if self.version.startswith('v_2.')
+            else df["Heat transfer coefficient of heat emitters"]
+        )
+        buildings["emitters_capacitance"] = np.array(
+            df["Thermal capacitance of emitters"]
+        )
+
+        return buildings
+
+
+    def _parse_controls_tpm(self):
+        # TODO: OlD implementation need to change
+        path = (
+            OLD_DATASET_PATH
+            + os.sep
+            + "CREST_data"
+            + os.sep
+            + "CREST_Demand_Model_v2.3.3.xlsm - HeatingControlsTPM.csv"
+        )
+        df_heating = pd.read_csv(
+            path, header=6, nrows=96, usecols=[2, 3, 4, 5]
+        )
+        thermostat["transitions cdf wd"] = np.cumsum(
+            df_heating.to_numpy()[:, :2].reshape((48, 2, 2)), axis=-1
+        )
+        thermostat["transitions cdf we"] = np.cumsum(
+            df_heating.to_numpy()[:, 2:].reshape((48, 2, 2)), axis=-1
+        )
+
+    def _parse_real_profiles_dict(self, profiles_type: str):
+        """Parse the real profiles.
+
+        Returns:
+            The appliance dictionary, of the form
+            {app_type: {app_name: array}}, such that it is easy to retrieve
+            the profiles based on the type of the appliances
+        """
+        if profiles_type != 'switchedON':
+            raise NotImplementedError('CREST only has switched on profiles.')
+        return {
+            'washingmachine': {
+                'CREST_washingmachine':
+                self._compute_washing_machine_power(np.arange(138, dtype=int))
+            },
+            'washer_dryer': {
+                'CREST_washer_dryer':
+                self._compute_washing_machine_power(np.arange(198, dtype=int))
+            }
+        }
+
+    def _compute_washing_machine_power(self, current_time):
+        """Washing machine power for a minute based simulation."""
+        power = np.zeros_like(current_time, dtype=float)
+        # define the values of power depending on n_times left, form CREST
+        power[current_time <= 8] = 73  # Start-up and fill
+        power[
+            np.logical_and(current_time > 8, current_time <= 29)
+        ] = 2056  # Heating
+        power[
+            np.logical_and(current_time > 29, current_time <= 81)
+        ] = 73  # Wash and drain
+        power[
+            np.logical_and(current_time > 81, current_time <= 92)
+        ] = 73  # Spin
+        power[
+            np.logical_and(current_time > 92, current_time <= 94)
+        ] = 250  # Rinse
+        power[
+            np.logical_and(current_time > 94, current_time <= 105)
+        ] = 73  # Spin
+        power[
+            np.logical_and(current_time > 105, current_time <= 107)
+        ] = 250  # Rinse
+        power[
+            np.logical_and(current_time > 107, current_time <= 118)
+        ] = 73  # Spin
+        power[
+            np.logical_and(current_time > 118, current_time <= 120)
+        ] = 250  # Rinse
+        power[
+            np.logical_and(current_time > 120, current_time <= 131)
+        ] = 73  # Spin
+        power[
+            np.logical_and(current_time > 131, current_time <= 133)
+        ] = 250  # Rinse
+        power[
+            np.logical_and(current_time > 133, current_time <= 138)
+        ] = 568  # Fast spin
+        power[
+            np.logical_and(current_time > 138, current_time <= 198)
+        ] = 2500  # Drying cycle
+        return power
