@@ -6,8 +6,15 @@ import warnings
 import datetime
 import pandas as pd
 import numpy as np
+from ...utils.sim_types import Subgroup
 from ...utils.sparse import SparseTPM
-from ...utils.parse_helpers import convert_states, states_to_tpms, states_to_transitions
+from ...utils.parse_helpers import (
+    convert_states,
+    get_initial_durations_pdfs,
+    states_to_tpms,
+    states_to_tpms_with_durations,
+    states_to_transitions,
+)
 
 MAX_PEOPLE_HOUSEHOLD = 6 + 1  # always need to get one above the real max pple
 
@@ -63,6 +70,7 @@ def get_mask_subgroup(
     n_residents=None, household_type=None,
     salary = None, hh_revenue=None,
     hh_work_type = None,
+    family_situation=None,
     life_situation=None, hh_mean_age=None, age=None, geburtsland=None,
     gender=None, household_position=None,
     is_travelling=None):
@@ -397,10 +405,10 @@ dic_home = {
 # convert to activity names
 GTOU_label_to_activity = {
     0 : 'only main activity',
-    110 : 'sleep',
-    120 : 'eat',
-    131 : 'wash self',
-    132 : 'sleep',
+    110 : 'sleeping',
+    120 : 'eating',
+    131 : 'self_washing',
+    132 : 'sleeping',
     139 : 'personal',
     210 : 'job',
     220 : 'job',
@@ -430,11 +438,11 @@ GTOU_label_to_activity = {
     363 : 'school',
     364 : 'school',
     369 : 'school',
-    411 : 'cook',
-    413 : 'cook',
-    412 : 'cook',
-    414 : 'cook',
-    419 : 'cook',
+    411 : 'cooking',
+    413 : 'cooking',
+    412 : 'cooking',
+    414 : 'cooking',
+    419 : 'cooking',
     421 : 'cleaning',
     422 : 'cleaning',
     423 : 'cleaning',
@@ -514,7 +522,7 @@ GTOU_label_to_activity = {
     814 : 'leisure',
     815 : 'leisure',
     819 : 'leisure',
-    820 : 'TV',
+    820 : 'watching_tv',
     830 : 'music',
     841 : 'computer/smartphone',
     842 : 'computer/smartphone',
@@ -705,6 +713,161 @@ GTOU_label_to_energy_activity = {
     999 : 'leisure'
 }
 
+# convert to activity names to this paper https://ieeexplore.ieee.org/document/8573766
+GTOU_label_to_Bottaccioli_act = {
+    0 : 'other',
+    110 : 'sleeping',
+    120 : 'other',
+    131 : 'self_washing',
+    132 : 'other',
+    139 : 'other',
+    210 : 'other',
+    220 : 'other',
+    230 : 'other',
+    241 : 'other',
+    242 : 'other',
+    243 : 'other',
+    244 : 'other',
+    245 : 'other',
+    249 : 'other',
+    311 : 'other',
+    312 : 'other',
+    313 : 'other',
+    314 : 'other',
+    315 : 'other',
+    317 : 'other',
+    319 : 'other',
+    321 : 'other',
+    329 : 'other',
+    330 : 'other',
+    341 : 'other',
+    349 : 'other',
+    353 : 'studying',
+    354 : 'studying',
+    361 : 'other',
+    362 : 'other',
+    363 : 'other',
+    364 : 'other',
+    369 : 'other',
+    411 : 'cooking',
+    413 : 'dishwashing',
+    412 : 'cooking',
+    414 : 'cooking',
+    419 : 'cooking',
+    421 : 'cleaning',
+    422 : 'cleaning',
+    423 : 'other',
+    429 : 'other',
+    431 : 'laundry',
+    432 : 'ironing',
+    433 : 'other',
+    434 : 'other',
+    439 : 'other',
+    441 : 'other',
+    442 : 'other',
+    443 : 'other',
+    444 : 'other',
+    445 : 'other',
+    446 : 'other',
+    449 : 'other',
+    451 : 'other',
+    452 : 'other',
+    453 : 'other',
+    454 : 'other',
+    455 : 'other',
+    459 : 'other',
+    461 : 'other',
+    464 : 'other',
+    465 : 'other',
+    466 : 'other',
+    469 : 'other',
+    471 : 'other',
+    472 : 'other',
+    473 : 'other',
+    474 : 'other',
+    475 : 'other',
+    476 : 'other',
+    479 : 'other',
+    480 : 'other',
+    491 : 'other',
+    492 : 'other',
+    499 : 'other',
+    510 : 'other',
+    520 : 'other',
+    531 : 'other',
+    532 : 'other',
+    539 : 'other',
+    611 : 'other',
+    612 : 'other',
+    621 : 'other',
+    622 : 'other',
+    623 : 'other',
+    624 : 'other',
+    625 : 'other',
+    626 : 'other',
+    627 : 'other',
+    629 : 'other',
+    630 : 'other',
+    641 : 'other',
+    642 : 'other',
+    649 : 'other',
+    711 : 'other',
+    712 : 'other',
+    713 : 'other',
+    715 : 'other',
+    716 : 'other',
+    717 : 'other',
+    719 : 'other',
+    730 : 'other',
+    740 : 'other',
+    752 : 'other',
+    759 : 'other',
+    761 : 'other',
+    762 : 'other',
+    763 : 'electronics',
+    769 : 'other',
+    790 : 'other',
+    811 : 'other',
+    812 : 'other',
+    813 : 'other',
+    814 : 'other',
+    815 : 'other',
+    819 : 'other',
+    820 : 'watching_tv',
+    830 : 'music',
+    841 : 'electronics',
+    842 : 'electronics',
+    843 : 'electronics',
+    844 : 'electronics',
+    849 : 'electronics',
+    921 : 'other',
+    922 : 'other',
+    923 : 'other',
+    929 : 'other',
+    931 : 'other',
+    934 : 'other',
+    939 : 'other',
+    941 : 'other',
+    945 : 'other',
+    946 : 'other',
+    947 : 'other',
+    948 : 'other',
+    949 : 'other',
+    951 : 'other',
+    952 : 'other',
+    953 : 'other',
+    959 : 'other',
+    961 : 'other',
+    962 : 'other',
+    969 : 'other',
+    970 : 'other',
+    980 : 'other',
+    991 : 'other',
+    992 : 'other',
+    997 : 'other',
+    998 : 'other',
+    999 : 'other'
+}
 # convert to activity names to the CREST consuming activities
 GTOU_label_to_CREST_act = {
     0 : '-',
@@ -1406,14 +1569,9 @@ def states_to_occupancy(primary_states, secondary_states, initial_occupancy, fin
 
     for i, (prim_state, sec_state, home_rating) in enumerate(zip(primary_states.T, secondary_states.T, home_ratings.T)):
 
-
-
         # only update the new states after a travel has been done
         mask_travelling = is_transportation(prim_state) | is_transportation(sec_state) # travelling is occurring
 
-
-
-
         # people who leave are not there anymore
         mask_leave = ~mask_was_travelling & mask_travelling
         mask_at_home_before_last_travel[mask_leave] = current_occupancy[mask_leave] # save state of before last travel
@@ -1571,9 +1729,106 @@ def get_active_occupancy(subgroup_kwargs):
 
     return np.sum(hh_states, axis=0)
 
+def get_tpms_activity(
+    subgroup: Subgroup,
+    activity_dict=GTOU_label_to_activity,
+    first_tpm_modification_algo='nothing',
+    add_away_state=True,
+    add_durations=False,
+    use_sec_states=True,
+):
+    """Get the tpms of a desired activity dictionray.
 
-def get_data_4states(subgroup_kwargs, first_tpm_modification_algo='last',):
-        # gets the concerned households
+    Attributes:
+        add_away_state:
+            Wether to replace the states by 'away' when there is
+            no one.
+        use_sec_states:
+            Wheter to use secondary states as well, to replace
+            the 'other' activity when a secondary state is not other
+
+    Does NOT group by households.
+    """
+    mask_subgroup = get_mask_subgroup( **subgroup)
+
+    raw_states = primary_states[mask_subgroup]
+
+    if add_away_state:
+        raw_states[~occ[mask_subgroup]] = 0
+        activity_dict = activity_dict.copy()
+        activity_dict[0] = 'away'
+
+
+    states, states_label = convert_states(raw_states, activity_dict)
+
+    states_label[states_label=='-'] = 'other'
+
+    if use_sec_states:
+        # Gets the secondary states
+        raw_sec_states = primary_states[mask_subgroup]
+        sec_states, sec_states_label = convert_states(
+            raw_sec_states, activity_dict
+        )
+        sec_states_label[sec_states_label=='-'] = 'other'
+        # Find where to replace
+        mask_can_replace = (
+            (states_label[states] == 'other')
+            & (sec_states_label[sec_states] != 'other')
+        )
+        # Recontruct a states array with the secondary states
+        raw_act_array = states_label[states]
+        raw_act_array[mask_can_replace] = sec_states_label[sec_states][mask_can_replace]
+        # Finds back in states format
+        states, states_label = convert_states(raw_act_array)
+
+
+    # Adds missing activities from dict to states labels
+    all_activities = np.array(list(activity_dict.values()))
+    missing_act = all_activities[~np.isin(all_activities, states_label)]
+    states_label = np.concatenate((states_label, missing_act))
+
+    # get the pdf of the initial distribution and save it
+    initial_counts = np.bincount( states[:,0], minlength=len(states_label))
+    initial_pdf = initial_counts/ np.sum(initial_counts)
+
+    dict_legend = {}
+    dict_legend['number of persons diaries'] = int(np.sum(mask_subgroup))
+    dict_legend['subgroup_kwargs'] = subgroup
+    dict_legend['cration date'] = str(datetime.datetime.now())
+    dict_legend['first_tpm_modification_algo'] = first_tpm_modification_algo
+
+    if add_durations:
+
+        tpm, duration, duration_with_previous = states_to_tpms_with_durations(
+            states,
+            first_tpm_modification_algo=first_tpm_modification_algo,
+            labels=states_label
+        )
+        # PDFs that should depend on the duration at start depending on
+        # the first state shape = n_states, n_times
+        intial_durations_pdf = get_initial_durations_pdfs(states)
+
+        return (
+            tpm, duration, states_label,
+            initial_pdf, intial_durations_pdf, dict_legend
+        )
+    else:
+        # Return the standard tpms algo
+        tpm = states_to_tpms(
+            states, first_tpm_modification_algo=first_tpm_modification_algo,
+            labels=states_label
+        )
+
+        return tpm, states_label, initial_pdf, dict_legend
+
+
+
+def get_data_4states(
+    subgroup_kwargs,
+    first_tpm_modification_algo='last',
+    add_durations=False
+):
+    # gets the concerned households
     mask_subgroup = get_mask_subgroup( **subgroup_kwargs)
 
     hh_states = group_in_household_4states(
@@ -1583,17 +1838,7 @@ def get_data_4states(subgroup_kwargs, first_tpm_modification_algo='last',):
     )
 
     states, states_label = convert_states(hh_states)
-    # Adds any missing states for the 4 states model
-    # n_res = subgroup_kwargs['n_residents']
 
-    # all_4_states = np.asarray([
-    #     i*10 + np.arange(0, n_res+1) for i in range(n_res+1)
-    # ]).reshape(-1)
-    # states_label = np.concatenate((  # Finds and adds the missing
-    #     states_label, all_4_states[~np.isin(all_4_states, states_label)]
-    # ))
-
-    tpm = states_to_tpms(states, first_tpm_modification_algo='last')
 
     # get the pdf of the initial distribution and save it
     initial_counts = np.bincount( states[:,0], minlength=len(states_label))
@@ -1606,8 +1851,26 @@ def get_data_4states(subgroup_kwargs, first_tpm_modification_algo='last',):
     dict_legend['cration date'] = str(datetime.datetime.now())
     dict_legend['first_tpm_modification_algo'] = first_tpm_modification_algo
 
+    if add_durations:
 
-    return tpm, states_label, initial_pdf, dict_legend
+        tpm, duration, duration_with_previous = states_to_tpms_with_durations(
+            states, first_tpm_modification_algo=first_tpm_modification_algo
+        )
+        # PDFs that should depend on the duration at start depending on
+        # the first state shape = n_states, n_times
+        intial_durations_pdf = get_initial_durations_pdfs(states)
+
+        return (
+            tpm, duration,  states_label,
+            initial_pdf, intial_durations_pdf, dict_legend
+        )
+    else:
+        # Return the standard tpms algo
+        tpm = states_to_tpms(
+            states, first_tpm_modification_algo=first_tpm_modification_algo
+        )
+
+        return tpm, states_label, initial_pdf, dict_legend
 
 def get_data_sparse9states(subgroup_kwargs):
 
@@ -1754,5 +2017,3 @@ def create_data_activity_profile(
             out_dict[demod_act][:, n_occ] = prob_at_least_one_occ_perfoming_act
 
     return out_dict
-
-
