@@ -3,11 +3,9 @@ from __future__ import annotations
 from datetime import timedelta
 import datetime
 import inspect
-from os.path import isfile
 from ..utils.parse_helpers import lists_to_numpy_array, make_jsonable
 from ..utils.sim_types import AppliancesDict, StateLabels, Subgroup, Subgroups, TPM, TPMs
-from ..utils.appliances import get_ownership_from_dict
-from ..utils.subgroup_handling import subgroup_string
+from ..utils.subgroup_handling import remove_time_attributues, subgroup_string
 
 import os
 import json
@@ -21,6 +19,7 @@ import pandas as pd
 
 
 DATASET_PATH = os.path.dirname(__file__)
+GITHUB_REPO_URL = 'https://github.com/epfl-herus/demod/tree/main/demod'
 
 
 class DatasetLoader:
@@ -72,10 +71,6 @@ class DatasetLoader:
                 parsed data will contain several version
                 None if there is only a single version.
             clear_parsed_data: Whether to clear the parsed data.
-
-
-        Returns:
-            Any: [description]
         """
         if not hasattr(self, "DATASET_NAME"):
             raise ValueError(
@@ -90,8 +85,17 @@ class DatasetLoader:
         self.raw_path = os.path.join(self.DATASET_PATH, "raw_data")
         self.parsed_path = os.path.join(self.DATASET_PATH, "parsed_data")
 
+        # Creates the directories if they do not exist
+        if not os.path.exists(self.parsed_path):
+            os.mkdir(self.parsed_path)
+        if not os.path.exists(self.raw_path):
+            os.mkdir(self.raw_path)
+
         if version is not None:
             self.parsed_path = os.path.join(self.parsed_path, version)
+            if not os.path.isdir(self.parsed_path):
+                os.mkdir(self.parsed_path)
+
         self.version = version
 
         if clear_parsed_data:
@@ -167,8 +171,8 @@ class DatasetLoader:
 
     def _warn_could_not_load_parsed(
         self,
-        exception_raised: Exception,
-        data_name: str,
+        exception_raised: FileNotFoundError,
+        data_name: str = None,
         warning_type: str = "warning",
     ):
         """Send a could not load parsed warning message.
@@ -177,9 +181,11 @@ class DatasetLoader:
         loading the raw data.
 
         Args:
-            exception_raised: The exception that was raised during the
+            exception_raised: The FileNotFoundError
+                exception that was raised during the
                 parsing of the data
-            data_name: The name of the data that was loaded
+            data_name: Deprecated: The name of the data that was loaded
+                (it won't be used, only kept for compatibility)
             warning_type: The type of warning to raise.
                 Curently implemented:
                     - 'warning' : using the warning module
@@ -188,14 +194,21 @@ class DatasetLoader:
                 If warning_type is not registerd, will use 'print'.
                 Defaults to 'warning'.
         """
+
+        if not isinstance(exception_raised, FileNotFoundError):
+            raise ValueError(
+                "{} should be a FileNotFoundError.".format(exception_raised)
+            )
+
         warn_msg = "".join(
             (
                 "Could not load parsed data for '{}', due to: \n '{}'",
-                "with message: '{}'.\nGenerating now from raw_data.",
+                "of file: '{}'.\nGenerating now from raw_data.",
             )
         )
         msg = warn_msg.format(
-            data_name, type(exception_raised).__name__, exception_raised
+            self.DATASET_NAME, type(exception_raised).__name__,
+            exception_raised.filename
         )
 
         if (warning_type == "warning") or (warning_type == "all"):
@@ -387,6 +400,7 @@ class PopulationLoader(DatasetLoader):
             "{}.".format(type(self).__name__)
         )
 
+
 class ApplianceLoader(DatasetLoader):
     """Loader that provide methods for loading appliances data.
 
@@ -394,6 +408,8 @@ class ApplianceLoader(DatasetLoader):
     * :py:meth:`_parse_appliance_dict`
     """
 
+    _SPLIT_CHAR: str = '&'  # A special char used to split names for save
+
     def load_appliance_dict(self) -> AppliancesDict:
         """Load the appliance dictionary.
 
@@ -427,12 +443,65 @@ class ApplianceLoader(DatasetLoader):
 
         return lists_to_numpy_array(appliance_dict)
 
+    def load_real_profiles_dict(
+        self, profiles_type: str = 'full'
+    ) -> Dict[str, Dict[str, np.ndarray]]:
+        """Load a dictionary containing real load profiles.
+
+        Try to call self. :py:meth:`_parse_real_profiles_dict` if the
+        parsed data is not available.
+
+        Args:
+            profiles_type: The type of profile to load. Possibilities:
+
+                * 'full', the wholes profiles are returned
+                * 'switchedON', only profiles when appliances are ON
+                * 'switchedOFF', only profiles when appliances are OFF
+
+        Returns:
+            The appliance dictionary, of the form
+            {app_type: {app_name: array}}, such that it is easy to retrieve
+            the profiles based on the type of the appliances
+        """
+        app_file = self.parsed_path + os.sep + "loadprofiles_" + profiles_type
+        try:
+            parsed_dict = np.load(app_file + '.npz')
+
+            load_profiles_dict = {}
+
+            for name, load in parsed_dict.items():
+                app_type, profile_name = str.split(name, self._SPLIT_CHAR)
+                if app_type not in load_profiles_dict:
+                    load_profiles_dict[app_type] = {}
+                load_profiles_dict[app_type][profile_name] = load
+
+        except FileNotFoundError as err:
+            self._warn_could_not_load_parsed(err, app_file)
+
+            load_profiles_dict = self._parse_real_profiles_dict(profiles_type)
+
+            parsed_dict = {}
+
+            for app_type, names_dict in load_profiles_dict.items():
+                for name, load in names_dict.items():
+                    parsed_dict[app_type + self._SPLIT_CHAR + name] = load
+            # Compress as the load profiles might be quite heavy
+            np.savez_compressed(app_file, **parsed_dict)
+
+        return load_profiles_dict
+
     def _parse_appliance_dict(self):
         raise NotImplementedError(
             "'_parse_appliance_dict' requires overriding in "
             "{}.".format(type(self).__name__)
         )
 
+    def _parse_real_profiles_dict(self, profiles_type: str):
+        raise NotImplementedError(
+            "'_parse_real_profiles_dict' requires overriding in "
+            "{}.".format(type(self).__name__)
+        )
+
     def _parse_appliance_ownership_dict(
         self, subgroup: Subgroup
     ) -> Dict[str, float]:
@@ -459,9 +528,9 @@ class ApplianceLoader(DatasetLoader):
                 probabilities.
 
         Return:
-            pdf of ownership for each appliance
+            probability of ownership for each appliance
         """
-        subgroup_str = subgroup_string(subgroup)
+        subgroup_str = subgroup_string(remove_time_attributues(subgroup))
 
         # put in a dedicated folder
         folder_name = "appliance_ownership"
@@ -492,6 +561,178 @@ class ApplianceLoader(DatasetLoader):
             "{}.".format(type(self).__name__)
         )
 
+    def load_yearly_target_switchons(
+        self,
+        subgroup: Subgroup = {},
+    ) -> Dict[str, float]:
+        """Return the target of switchons in a year of each appliances.
+
+        A subgroup can be specified for datasets that differentiate
+        different subgroups.
+
+        :py:func:`~demod.utils.appliances.get_target_from_dict`
+        can then be used to sample the target number of yearly switchons
+        using an appliance dictionary.
+
+        Args:
+            subgroup: The subgroup of the desired targets number of
+                switchons.
+
+        Return:
+            Number of target switchons for each appliance type.
+        """
+        subgroup_str = subgroup_string(
+            # Time attributes don't change target of switchons
+            remove_time_attributues(subgroup)
+        )
+
+        # put in a dedicated folder
+        folder_name = "appliance_targets"
+        parsed_path_targets = self.parsed_path + os.sep + folder_name
+        if not os.path.exists(parsed_path_targets):
+            os.mkdir(parsed_path_targets)
+
+        file_name = folder_name + os.sep + "switchons__" + subgroup_str
+        file_path = os.path.join(self.parsed_path, file_name)
+
+        try:
+            with open(file_path, "r") as f:
+                targets_dict = dict(json.load(f))
+        except FileNotFoundError as err:
+            self._warn_could_not_load_parsed(err, file_name)
+            targets_dict = self._parse_yearly_target_switchons(subgroup)
+            with open(file_path, "w+") as f:
+                json.dump(make_jsonable(targets_dict), f, indent=2)
+
+        return targets_dict
+
+    def _parse_yearly_target_switchons(
+        self,
+        subgroup: Subgroup = None,
+    ) -> Union[np.ndarray, Dict[str, float]]:
+
+        raise NotImplementedError(
+            "'_parse_yearly_target_switchons' requires overriding in "
+            "{}.".format(type(self).__name__)
+        )
+
+    def load_yearly_target_consumption(
+        self,
+        subgroup: Subgroup = {},
+    ) -> Dict[str, float]:
+        """Return the target of consumption in a year of each appliances.
+
+        A subgroup can be specified for datasets that differentiate
+        different subgroups.
+
+        :Unit: : KwH/y
+
+        :py:func:`~demod.utils.appliances.get_target_from_dict`
+        can then be used to sample the target number of yearly consumption
+        using an appliance dictionary.
+
+        Args:
+            subgroup: The subgroup of the desired targets number of
+                consumption.
+
+        Return:
+            Number of target consumption for each appliance type.
+        """
+        subgroup_str = subgroup_string(
+            # Time attributes don't change target of consumption
+            remove_time_attributues(subgroup)
+        )
+
+        # put in a dedicated folder
+        folder_name = "appliance_targets"
+        parsed_path_targets = self.parsed_path + os.sep + folder_name
+        if not os.path.exists(parsed_path_targets):
+            os.mkdir(parsed_path_targets)
+
+        file_name = folder_name + os.sep + "consumption__" + subgroup_str
+        file_path = os.path.join(self.parsed_path, file_name)
+
+        try:
+            with open(file_path, "r") as f:
+                targets_dict = dict(json.load(f))
+        except FileNotFoundError as err:
+            self._warn_could_not_load_parsed(err, file_name)
+            targets_dict = self._parse_yearly_target_consumption(subgroup)
+            with open(file_path, "w+") as f:
+                json.dump(make_jsonable(targets_dict), f, indent=2)
+
+        return targets_dict
+
+    def _parse_yearly_target_consumption(
+        self,
+        subgroup: Subgroup = None,
+    ) -> Union[np.ndarray, Dict[str, float]]:
+
+        raise NotImplementedError(
+            "'_parse_yearly_target_consumption' requires overriding in "
+            "{}.".format(type(self).__name__)
+        )
+
+    def load_yearly_target_duration(
+        self,
+        subgroup: Subgroup = {},
+    ) -> Dict[str, float]:
+        """Return the target of duration in a year of each appliances.
+
+        The duration is in number of steps simulated when the appliance
+        should be on.
+
+        A subgroup can be specified for datasets that differentiate
+        different subgroups.
+
+        :Unit: : number of steps
+
+        :py:func:`~demod.utils.appliances.get_target_from_dict`
+        can then be used to sample the target number of yearly duration
+        using an appliance dictionary.
+
+        Args:
+            subgroup: The subgroup of the desired targets number of
+                duration.
+
+        Return:
+            Number of target duration for each appliance type.
+        """
+        subgroup_str = subgroup_string(
+            # Time attributes don't change target of duration
+            remove_time_attributues(subgroup)
+        )
+
+        # put in a dedicated folder
+        folder_name = "appliance_targets"
+        parsed_path_targets = self.parsed_path + os.sep + folder_name
+        if not os.path.exists(parsed_path_targets):
+            os.mkdir(parsed_path_targets)
+
+        file_name = folder_name + os.sep + "duration__" + subgroup_str
+        file_path = os.path.join(self.parsed_path, file_name)
+
+        try:
+            with open(file_path, "r") as f:
+                targets_dict = dict(json.load(f))
+        except FileNotFoundError as err:
+            self._warn_could_not_load_parsed(err, file_name)
+            targets_dict = self._parse_yearly_target_duration(subgroup)
+            with open(file_path, "w+") as f:
+                json.dump(make_jsonable(targets_dict), f, indent=2)
+
+        return targets_dict
+
+    def _parse_yearly_target_duration(
+        self,
+        subgroup: Subgroup = None,
+    ) -> Union[np.ndarray, Dict[str, float]]:
+
+        raise NotImplementedError(
+            "'_parse_yearly_target_duration' requires overriding in "
+            "{}.".format(type(self).__name__)
+        )
+
 
 class ClimateLoader(DatasetLoader):
     """Loader providing methods for loading climate data.
@@ -625,7 +866,7 @@ class ClimateLoader(DatasetLoader):
             climate_dict, a dictionary with the following possible keys
 
             * 'datetime': the time stored as numpy 'datetime64', only mandatory key. the datetime array should be in utc format time.
-            * 'temperature': the temperature of the air [C]
+            * 'outside_temperature': the temperature of the air [C]
             * 'radiation_diffuse': diffuse radiation at surface [W/m^2]
             * 'radiation_direct': direct radiation at surface [W/m^2]
             * 'radiation_global': global radiation at surface [W/m^2]
@@ -688,7 +929,7 @@ class LightingLoader(DatasetLoader):
 
     def load_fisher_lighting(self) -> Dict[str, Any]:
         """Load data for
-        :py:class:`~demod.simulators.lighting_simulators.FisherLighitingSimulator`
+        :py:class:`~demod.simulators.lighting_simulators.FisherLightingSimulator`
 
         Returns:
             Fisher lighting sim parameters dict.
@@ -951,4 +1192,4 @@ class HeatingLoader(DatasetLoader):
         raise NotImplementedError(
             "'_parse_thermostat_dict' requires overriding in "
             "{}.".format(type(self).__name__)
-        )
\ No newline at end of file
+        )
