@@ -7,19 +7,10 @@ implementations of simulations.
 """
 from __future__ import annotations
 
-
-import itertools
 import datetime
 from typing import List, NoReturn, Union, Callable
 
 import numpy as np
-from numpy.core.fromnumeric import shape
-
-from ..utils.monte_carlo import monte_carlo_from_cdf
-from ..utils.distribution_functions import check_valid_cdf
-
-
-
 
 
 def cached_getter(getter: GetMethod) -> GetMethod:
@@ -31,7 +22,8 @@ def cached_getter(getter: GetMethod) -> GetMethod:
     Args:
         getter: The getter method to decorate.
     """
-    def decorated_getter(self: Simulator, n_ieth_hh: Union[None,int]=None):
+
+    def decorated_getter(self: Simulator, n_ieth_hh: Union[None, int] = None):
         # add getter to the cache if not there
         if getter not in self._cache:
             self._cache[getter] = getter(self)
@@ -41,9 +33,9 @@ def cached_getter(getter: GetMethod) -> GetMethod:
 
         return self._cache[getter][n_ieth_hh]
 
-
     return decorated_getter
 
+
 class SimLogger():
     """Specialized logger for any Simulator object.
 
@@ -172,9 +164,11 @@ class SimLogger():
                 # for dictionary attributes, convert each value of the
                 # dict
                 return {
-                    key: np.asarray(val) for key, val in out.items()
+                    # Remove last point to compensate intialization record
+                    key: np.asarray(val)[:-1] for key, val in out.items()
                 }
-            return np.asarray(out)
+            # Remove last point to compensate intialization record
+            return np.asarray(out)[:-1]
         else:
             self._raise_unkown_attribute(attribute)
 
@@ -519,7 +513,8 @@ class Simulator():
         # clears the logger after initialization
         if self.logger:
             self.logger.clear()
-
+            # visit to store the first step
+            self.logger.visit_simulator(self)
 
 
     def step(self) -> None:
@@ -530,11 +525,11 @@ class Simulator():
         """
         # update the time
         self.current_time_step += 1
+        # clear the cache
+        self._cache.clear()
         # call the logger
         if self.logger:
             self.logger.visit_simulator(self)
-        # clear the cache
-        self._cache.clear()
 
 
 # Support for Typing in simulators.
@@ -600,8 +595,6 @@ class MultiSimulator(Simulator):
         Note:
             A future implmentation could implement parallelization.
         """
-        super().step()
-
         # Tracks the household offset when iterating over subsimulators
         hh_offset:int = 0
         for sim in self.simulators:
@@ -616,6 +609,8 @@ class MultiSimulator(Simulator):
             sim.step(*sub_args, **sub_kwargs)
             hh_offset += sim.n_households
 
+        super().step()
+
     def _assign_getters(self) -> None:
         """Assign getters to the multi simulator, using sub-simulator."""
         # Check the first sim get_ methods
