@@ -17,6 +17,7 @@ import math
 
 
 from .base_simulators import (
+    Callbacks,
     Simulator,
     MultiSimulator,
     TimeAwareSimulator,
@@ -236,7 +237,7 @@ class Crest4StatesModel(TimeAwareSimulator, MultiSimulator):
         **kwargs
     ):
         """Initialize a simulator for households of different number of
-        residents based on the CREST data
+        residents based on CREST 4 States model.
 
         Args:
             n_households (int): the total number of households to simulate
@@ -303,13 +304,11 @@ class Crest4StatesModel(TimeAwareSimulator, MultiSimulator):
         )
 
         super().initialize_starting_state(
-            initialization_time=datetime.time(0),  # CREST starts at midnight
+            initialization_time=self.data.refresh_time,
         )
 
-    def on_after_next_day(self) -> None:
-        """Update the CREST TPMs for the new daytype.
-
-        TODO Change that to a better data implementation
+    def on_after_refresh_time(self) -> None:
+        """Update the TPMs for the new daytype.
 
         Note:
             It was deduced that the change of TPM must be perform after
@@ -327,8 +326,8 @@ class Crest4StatesModel(TimeAwareSimulator, MultiSimulator):
                 )
             for sim in self.simulators:
                 subgroup['n_residents'] = sim.n_residents
-                tpms, _, _ = self.data.load_tpm(subgroup)
-                sim._set_tpm(tpms)
+                tpms, new_labels, _ = self.data.load_tpm(subgroup)
+                sim._set_tpm(tpms, new_labels=new_labels)
 
     def get_states_labeled(self):
 
@@ -336,7 +335,7 @@ class Crest4StatesModel(TimeAwareSimulator, MultiSimulator):
             [sim.state_labels[sim.current_states] for sim in self.simulators]
         )
 
-    @after_next_day_callback
+    @Callbacks.after_refresh_time
     def step(self) -> None:
         """Updates the states of the residents."""
         return super().step()
@@ -473,11 +472,11 @@ class CRESTAppliancesSimulator(AppliancesSimulator):
         """
         # first switch on
         # add a simple value that is the mean
-        self.n_times_left[
+        self.n_steps_left[
             indexes_household, indexes_appliance
         ] = self.appliances["mean cycle durations"][indexes_appliance]
         # adds the time till refresh is possible
-        self.n_times_till_refresh[indexes_household, indexes_appliance] = (
+        self.n_steps_till_refresh[indexes_household, indexes_appliance] = (
             self.appliances["mean cycle durations"][indexes_appliance]
             + self.appliances["delay after cycle"][indexes_appliance]
         )
@@ -518,9 +517,9 @@ class CRESTAppliancesSimulator(AppliancesSimulator):
             volumes / self.appliances["mean cycle power"][indexes_appliance]
         )  # power is flow for water appliances
         # water fixtures duration
-        self.n_times_left[indexes_household, indexes_appliance] = durations
+        self.n_steps_left[indexes_household, indexes_appliance] = durations
         # adds the time till refresh is possible
-        self.n_times_till_refresh[indexes_household, indexes_appliance] = (
+        self.n_steps_till_refresh[indexes_household, indexes_appliance] = (
             durations + self.appliances["delay after cycle"][indexes_appliance]
         )
 
@@ -542,24 +541,24 @@ class CRESTAppliancesSimulator(AppliancesSimulator):
         )
 
         # finish the time remaining for the ones that were being used
-        self.n_times_left[mask_need_switchoff] = 0
+        self.n_steps_left[mask_need_switchoff] = 0
         # start or continue a refresh period for them
-        self.n_times_till_refresh[mask_need_switchoff] = np.minimum(
-            self.n_times_till_refresh[mask_household_need_switchoff][
+        self.n_steps_till_refresh[mask_need_switchoff] = np.minimum(
+            self.n_steps_till_refresh[mask_household_need_switchoff][
                 :, mask_app_need_switchoff
             ],
             self.appliances["delay after cycle"][mask_app_need_switchoff],
         ).reshape(-1)
 
     def get_current_power_consumptions(self):
-        power_consumptions = np.zeros_like(self.n_times_left, dtype=float)
+        power_consumptions = np.zeros_like(self.n_steps_left, dtype=float)
         # check the appliances that are on or off and determine the values of power for on and off
         power_consumptions += (
-            (self.n_times_left == 0)
+            (self.n_steps_left == 0)
             * self.appliances["standby power"]
             * self.available_appliances
         )
-        power_consumptions += (self.n_times_left > 0) * self.appliances[
+        power_consumptions += (self.n_steps_left > 0) * self.appliances[
             "mean cycle power"
         ]
 
@@ -570,7 +569,7 @@ class CRESTAppliancesSimulator(AppliancesSimulator):
         power_consumptions[
             mask_available, index
         ] = self._compute_washing_machine_power(
-            self.n_times_left[mask_available, index], "WASHING_MACHINE"
+            self.n_steps_left[mask_available, index], "WASHING_MACHINE"
         )
 
         index = self.appliances["name"] == "WASHER_DRYER"
@@ -578,7 +577,7 @@ class CRESTAppliancesSimulator(AppliancesSimulator):
         power_consumptions[
             mask_available, index
         ] = self._compute_washing_machine_power(
-            self.n_times_left[mask_available, index], "WASHER_DRYER"
+            self.n_steps_left[mask_available, index], "WASHER_DRYER"
         )
 
         # the water appliances do not consume electricity
@@ -594,20 +593,20 @@ class CRESTAppliancesSimulator(AppliancesSimulator):
         return power_consumptions
 
     def get_current_water_consumptions(self):
-        water_consumptions = np.zeros_like(self.n_times_left, dtype=float)
+        water_consumptions = np.zeros_like(self.n_steps_left, dtype=float)
         # check the appliances that are on or off and determine the values of power for on and off
         water_consumptions += (
-            (self.n_times_left <= 0)
+            (self.n_steps_left <= 0)
             * self.appliances["standby power"]
             * self.available_appliances
         )
         # water takes into account short duration events (less than 1 min)
         water_consumptions += (
-            self.n_times_left
-            * ((self.n_times_left > 0) & (self.n_times_left < 1))
+            self.n_steps_left
+            * ((self.n_steps_left > 0) & (self.n_steps_left < 1))
             * self.appliances["mean cycle power"]
         )
-        water_consumptions += (self.n_times_left >= 1) * self.appliances[
+        water_consumptions += (self.n_steps_left >= 1) * self.appliances[
             "mean cycle power"
         ]
 
@@ -635,15 +634,15 @@ class CRESTAppliancesSimulator(AppliancesSimulator):
         SPECIFIC_HEAT_CAPACITY_WATER = 4200.0
         return SPECIFIC_HEAT_CAPACITY_WATER * dblM_w
 
-    def _compute_washing_machine_power(self, n_times_left, name):
+    def _compute_washing_machine_power(self, n_steps_left, name):
         if name == "WASHING_MACHINE":
-            current_time = 138 - n_times_left
+            current_time = 138 - n_steps_left
         elif name == "WASHER_DRYER":
-            current_time = 198 - n_times_left
+            current_time = 198 - n_steps_left
         else:
             raise ValueError(name + " is not a valid name")
 
-        power = np.zeros_like(n_times_left, dtype=float)
+        power = np.zeros_like(n_steps_left, dtype=float)
         # define the values of power depending on n_times left, form CREST
         power[current_time <= 8] = 73  # Start-up and fill
         # Heating
@@ -683,7 +682,7 @@ class CRESTAppliancesSimulator(AppliancesSimulator):
         ] = 2500  # Drying cycle
 
         # standby
-        power[n_times_left == 0] = self.appliances["standby power"][
+        power[n_steps_left == 0] = self.appliances["standby power"][
             self.appliances["name"] == name
         ]
 
@@ -1356,87 +1355,7 @@ class HeatingSystemSimulator(Simulator):
 
         return buildings, emitters
 
-    def CRESTDATA_read_heating_system(self):
-        path = (
-            OLD_DATASET_PATH
-            + os.sep
-            + "CREST_data"
-            + os.sep
-            + "CREST_Demand_Model_v2.3.3.xlsm - PrimaryHeatingSystems.csv"
-        )
-        df = pd.read_csv(path, header=1, nrows=5, skiprows=[2, 3])
 
-        heating_systems = {}
-        heating_systems["equipped probs"] = np.array(
-            [
-                float(i[:-1]) / 100
-                for i in df["Proportion of dwellings with this heating system"]
-            ]
-        )  # remove the percent sign
-        heating_systems["heating type"] = np.array(df["Type of heating unit"])
-        heating_systems["regular/combi"] = np.array(df["Type of system"])
-        heating_systems["fuel type"] = np.array(df["Type of fuel"])
-        heating_systems["fuel flow rate"] = np.array(
-            df["Fuel flow rate (nominal)"]
-        )
-        heating_systems["heat output [W]"] = np.array(
-            df["Heat output of unit"]
-        )
-        heating_systems["standby power"] = np.array(df["Standby power"])
-        heating_systems["pump power"] = np.array(df["Pump power"])
-        heating_systems["thermal efficiency"] = np.array(
-            [float(i[:-1]) for i in df["Thermal efficiency"]]
-        )  # remove the percent sign
-        heating_systems["cylinder volume"] = np.array(
-            df["DHW cylinder volume"]
-        )
-        heating_systems["thermal resistance loss"] = np.array(
-            df["DHW Tank Heat Loss"]
-        )
-
-        return heating_systems
-
-    def GERMANY_read_heating_system(self):
-
-        path = OLD_DATASET_PATH + os.sep + "GermanTOU" + os.sep + "inputs.ods"
-        df = pd.read_excel(
-            path, header=1, skiprows=[2, 3], sheet_name="Heating Systems"
-        )
-
-        heating_systems = {}
-        heating_systems["equipped probs"] = np.array(
-            df["Proportion of dwellings with this heating system"]
-        )  # remove the percent sign
-        heating_systems["heating type"] = np.array(df["Type of heating unit"])
-        heating_systems["regular/combi"] = np.array(df["Type of system"])
-        heating_systems["fuel type"] = np.array(df["Type of fuel"])
-        heating_systems["fuel flow rate"] = np.array(
-            df["Fuel flow rate (nominal)"]
-        )
-        heating_systems["heat output [W]"] = np.array(
-            df["Heat output of unit"]
-        )
-        heating_systems["standby power"] = np.array(df["Standby power"])
-        heating_systems["pump power"] = np.array(df["Pump power"])
-        heating_systems["thermal efficiency"] = np.array(
-            df["Thermal efficiency"]
-        )  # remove the percent sign
-        heating_systems["cylinder volume"] = np.array(
-            df["DHW cylinder volume"]
-        )
-        heating_systems["thermal resistance loss"] = np.array(
-            df["DHW Tank Heat Loss"]
-        )
-
-        return heating_systems
-
-    def read_heating_system(self, data):
-        if data == "CREST":
-            return self.CRESTDATA_read_heating_system()
-        elif data == "GTOU":
-            return self.GERMANY_read_heating_system()
-        else:
-            raise ValueError("Unknonw data :" + data)
 
     def read_buildings(self, data):
         if data == "CREST":
