@@ -288,6 +288,12 @@ class Inputs:
             :Unit: people, :py:obj:`int`
             :Related Getter: :py:meth:`~.Sim.get_active_occupancy`
 
+        activities_dict: The dictionary containing the activities of the
+            residents.
+
+            :Unit: people, :py:obj:`int`
+            :Related Getter: :py:meth:`~.Sim.get_activities`
+
         thermal_gains: The heat produced by the subjects of the
             simulator.
 
@@ -382,6 +388,18 @@ class Inputs:
             :Unit: binary, :py:obj:`bool`
             :Related Getter: :py:meth:`~.Sim.get_controls`
 
+        external_outside_temperature: Input from an external component for
+            the outside temperature.
+
+            :Unit: temperature, :math:`[^{\circ} C]`
+            :Related Getter: :py:meth:`~.Sim.get_outside_temperature`
+
+        external_irradiance: Input from an external component for
+            the irradiance.
+
+            :Unit: irradiance, :math:`[W/m^2]`
+            :Related Getter: :py:meth:`~.Sim.get_irradiance`
+
         external_target_temperature: Input from an external component for
             the room temperatures.
 
@@ -464,28 +482,96 @@ class Params:
             for simulators that can update their attributes depending on
             the datetime.
 
-        start_datetime: A datetime.datetime object specifiying the moment of
+        start_datetime: A `datetime.datetime object
+            <https://docs.python.org/3/library/datetime.html#datetime-objects>`_
+            specifiying the moment of
             the start of the simulation.
 
-        step_size: A datetime.timedelta object specifying the duration
-            of a simulator step.
+        step_size: A `datetime.timedelta object
+            <https://docs.python.org/3/library/datetime.html#timedelta-objects>`_
+            specifying the duration of a simulator step.
 
-        subgroup: A dictionary containing information on a certain type
+        subgroup: A python dictionary containing information on a
+            certain types
             of people or households from the dataset.
+            a subgroup has different key-value pairs that specifiy
+            socio-techno-economico-types of the population.
+            The possible keys used in demod's subgroups are defined
+            below.
 
-            :Allowed keys:
+            .. note::
+                Combination of different keys is based on intersection, which
+                means that only households/persons that respect all
+                the conditions implied by the keys will be included.
+
+            :Possible keys:
 
                 * subgroup['n_residents']
                     int, The number of residents in the household
+                * subgroup['household_type']
+                    int, The number of residents in the household
+                    From :py:class:`~demod.datasets.GermanTOU.loader.GTOU`
 
+                    * 1 = One person household
+                    * 2 = Couple without kid
+                    * 3 = Single Parent with at least one kid under 18
+                      and the other under 27
+                    * 4 = Couple with at least one kid under 18
+                      and the other under 27
+                    * 5 = Others
+
+                * subgroup['life_situation']
+                    int, The life situation of the persons.
+                    From :py:class:`~demod.datasets.GermanTOU.loader.GTOU`
+
+                    * 1 = Self-employed, freelancer, farmer, family worker
+                    * 2 = Employee, worker, civil servant, judge,
+                      temporary / professional soldier, voluntary social /
+                      ecological / cultural year, voluntary military service,
+                      federal voluntary service
+                    * 3 = Trainee (also intern, volunteer)
+                    * 4 = Partial Work Time (work or retirement)
+                    * 5 = On parental leave (with an employment contract
+                      that has not been terminated)
+                    * 6 = Student, Pupil
+                    * 7 = Unemployed
+                    * 8 = Retired or early retirement
+                    * 9 = Permanently disabled
+                    * 10 = Housewife / househusband/ houseparent
+                    * 11 = Not gainfully employed for other reasons
+
+                    .. note::
+                        This works better with single persons households
+                        in general.
+
+                * subgroup['household_position']
+                    int, The position of the person in the household or family.
+                    From :py:class:`~demod.datasets.GermanTOU.loader.GTOU`
+
+                    * 1 = Main revenue maker
+                    * 2 = Partner or Spouse
+                    * 3 = Kid
+
+                    .. note::
+                        This works better with single persons households
+                        or person subgroup.
+
+
+                * subgroup['age']
+                    int or tuple, The age of the participants, if tuple
+                    returns all inside interval.
+                * subgroup['hh_mean_age']
+                    tuple, The mean age of all memeber of the household.
+                    Includes all households with mean age inside the
+                    interval given as tuple.
+                * subgroup['gender']
+                    int, 1 = man, 2 = woman, 3 = other
                 * subgroup['weekday']
                     int or list, The day of the week (1 to 7). If list,
-                    include all days in list.
+                    include all days in list. (1 = Monday, 7 = Sunday).
                 * subgroup['quarter']
                     int or list, The quarter of the year (1 to 4).
 
-            When multiple keys are given, the subgroup is the Union,
-            intersection of all of them.
 
         subgroups: A list of :py:attr:`subgroup`.
 
@@ -530,7 +616,7 @@ class Params:
                 in :math:`[l/min]`
             * appliances_dict['inactive_switch_off']
                 *bool*, Whether the appliance should switch off if the
-                occpupants leave or sleep.
+                occupants stop the activity, leave or sleep.
             * appliances_dict['equipped_prob']
                 *float*, Probability that a household is equipped with
                 this appliance.
@@ -559,10 +645,14 @@ class Params:
 
         appliance_type: A string that represent the type of appliance.
             If the name is followed by '_2', it signifies that the
-            appliance is a secondary appliance of that value.
+            appliance is a secondary appliance of that type.
             Nested levels indicate subtypes.
             Nested name should always finish with '_basename'.
             Types are very important for datset compatibilities.
+            The current naming is purely arbitrary, but it could be
+            changed in the future for compatibilty with an official
+            naming convention from other sources.
+
             Possible values:
 
             * 'freezer'
@@ -586,13 +676,24 @@ class Params:
                 * 'cd_speaker'
                 * 'radio_speaker'
                 * 'hifi_speaker'
+
             * 'tv'
+
+                * 'crt_tv'
+                * 'tft_tv'
+                * 'lcd_tv'
+                * 'led_tv'
+                * 'dlp_tv'
+                * 'plasma_tv'
+                * 'oled_tv'
+
             * 'box'
 
                 * 'tv_box'
                 * 'internet_box'
                 * 'dual_box'
                 Internet + TV
+                * 'wifi_box'
 
             * 'console'
 
@@ -603,14 +704,34 @@ class Params:
             * 'computer'
 
                 * 'laptop_computer'
-                * 'fixed_computer'
+                * 'fixed_computer' (Desktop)
                 * 'gaming_computer'
 
+            * 'monitor'
+
+                * 'crt_monitor'
+                * 'tft_monitor'
+                * 'lcd_monitor'
+                * 'led_monitor'
+                * 'dlp_monitor'
+                * 'touchscreen_monitor'
+                * 'plasma_monitor'
+                * 'oled_monitor'
+
+            * 'projector'
+
             * 'printer'
 
                 * 'fax_printer'
 
+            * 'lamp'
+
+                * 'christmas_lamp'
+
             * 'clock'
+
+                * 'alarm_clock'
+
             * 'hob'
 
                 * 'electric_hob'
@@ -619,6 +740,12 @@ class Params:
             * 'oven'
             * 'microwave'
             * 'kettle'
+            * 'waterboiler'
+            * 'coffemachine'
+
+                * 'beans_coffemachine'
+                * 'capsule_coffemachine'
+
             * 'toaster'
             * 'fitnessmachine'
             * 'dishwasher'
@@ -627,7 +754,7 @@ class Params:
 
                 * 'tumble_dryer'
                 * 'washer_dryer'
-                Washing Machine that do both washing and drying.
+                    Washing Machine that do both washing and drying.
 
             * 'iron'
             * 'heater'
@@ -635,11 +762,11 @@ class Params:
                 * 'water_heater'
 
                     * 'des_water_heater'
-                    Domestic electric storage
-                    water heater
+                        Domestic electric storage
+                        water heater
 
                     * 'einst_water_heater'
-                    Instant electric hot water
+                        Instant electric hot water
 
             * 'cleaner'
 
@@ -683,12 +810,16 @@ class Params:
             * 'electronics'
                 General purpose for using electronics. (Computer, consoles)
 
-
         equipped_sampling_algo: A string representing the algorithm to
             use for sampling the appliances of a house.
             Possible values detailed in
             :py:meth:`~demod.simulators.appliance_simulators.AppliancesSimulator.sample_available_appliances`
 
+        real_profiles_algo: A string representing the algorithm to
+            use for sampling the real load profiles of appliances.
+            Possible values detailed in
+            :py:meth:`~demod.simulators.appliance_simulators.AppliancesSimulator.sample_real_load_profiles`
+
         bulbs_sampling_algo: The algorithm to sample the lighting bulbs
                 installed in each house. See
                 :py:meth:`~demod.simulators.lighting_simulators.CrestLightingSimulator.sample_bulbs_configuration`.
@@ -726,6 +857,9 @@ class Params:
         initial_active_occupancy: The active occupancy at the start of
             the simulation. Used for the initialization of a simulator.
 
+        initial_activities_dict: The activities at the start of
+            the simulation. Used for the initialization of a simulator.
+
         initial_clearness: The clearness at the start of the simulation.
 
         initial_irradiance: The irradiance at the start of the simulation.
