@@ -21,6 +21,10 @@ ALGO_REQUIRES_LOADING_METHOD = (
     " from dataset. Could not find '{loading_method}' in '{dataset}'."
 )
 
+USE_OTHER_ALGOS_FOR_ALGONAME = (
+    "Try using {other_algos} as {algo_name} instead"
+)
+
 UNKOWN_POPULATION_TYPE = (
     "Unkown population type '{population_type}' for dataset"
     " {dataset}."
@@ -38,4 +42,4 @@ UNIMPLEMENTED_ALGO_IN_METHOD = (
 DATASET_CANNOT_DISTINGUISH_ON_SUBGROUPS = (
     "{dataset} cannot distinguish {not_distinguishable}  based on subgroups.\n"
     "{not_distinguishable} will be the same for all subgroups."
-)
\ No newline at end of file
+)
